@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2006 David Zeuthen <davidz@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
@@ -39,14 +39,16 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 
+#include <libhal-gdevice.h>
+#include <libhal-gmanager.h>
+#include <libdbus-proxy.h>
+
 #include "gpm-brightness-kbd.h"
 #include "gpm-conf.h"
 #include "gpm-common.h"
 #include "gpm-debug.h"
-#include "gpm-hal.h"
 #include "gpm-light-sensor.h"
 #include "gpm-marshal.h"
-#include "gpm-proxy.h"
 
 #define DIM_INTERVAL		10 /* ms */
 
@@ -64,9 +66,8 @@ struct GpmBrightnessKbdPrivate
 	guint			 levels;
 	gchar			*udi;
 	GpmConf			*conf;
-	GpmHal			*hal;
 	GpmLightSensor		*sensor;
-	GpmProxy		*gproxy;
+	DbusProxy		*gproxy;
 };
 
 enum {
@@ -96,7 +97,7 @@ gpm_brightness_kbd_get_hw (GpmBrightnessKbd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	proxy = gpm_proxy_get_proxy (brightness->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (brightness->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected to HAL");
 		return FALSE;
@@ -104,7 +105,7 @@ gpm_brightness_kbd_get_hw (GpmBrightnessKbd *brightness,
 
 	ret = dbus_g_proxy_call (proxy, "GetBrightness", &error,
 				 G_TYPE_INVALID,
-				 G_TYPE_INT, brightness_level_hw,
+				 G_TYPE_UINT, brightness_level_hw,
 				 G_TYPE_INVALID);
 	if (error) {
 		gpm_debug ("ERROR: %s", error->message);
@@ -138,7 +139,7 @@ gpm_brightness_kbd_set_hw (GpmBrightnessKbd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	proxy = gpm_proxy_get_proxy (brightness->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (brightness->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected to HAL");
 		return FALSE;
@@ -420,7 +421,7 @@ gpm_brightness_kbd_up (GpmBrightnessKbd *brightness)
 
 	percentage = gpm_discrete_to_percent (brightness->priv->current_hw,
 					      brightness->priv->levels);
-	g_debug ("emitting brightness-changed (%i)", percentage);
+	gpm_debug ("emitting brightness-changed (%i)", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_CHANGED], 0, percentage);
 
 	return TRUE;
@@ -456,7 +457,7 @@ gpm_brightness_kbd_down (GpmBrightnessKbd *brightness)
 
 	percentage = gpm_discrete_to_percent (brightness->priv->current_hw,
 					      brightness->priv->levels);
-	g_debug ("emitting brightness-changed (%i)", percentage);
+	gpm_debug ("emitting brightness-changed (%i)", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_CHANGED], 0, percentage);
 
 	return TRUE;
@@ -495,9 +496,6 @@ gpm_brightness_kbd_finalize (GObject *object)
 	if (brightness->priv->gproxy != NULL) {
 		g_object_unref (brightness->priv->gproxy);
 	}
-	if (brightness->priv->hal != NULL) {
-		g_object_unref (brightness->priv->hal);
-	}
 	if (brightness->priv->conf != NULL) {
 		g_object_unref (brightness->priv->conf);
 	}
@@ -526,10 +524,10 @@ gpm_brightness_kbd_class_init (GpmBrightnessKbdClass *klass)
 			      G_STRUCT_OFFSET (GpmBrightnessKbdClass, brightness_changed),
 			      NULL,
 			      NULL,
-			      g_cclosure_marshal_VOID__INT,
+			      g_cclosure_marshal_VOID__UINT,
 			      G_TYPE_NONE,
 			      1,
-			      G_TYPE_INT);
+			      G_TYPE_UINT);
 
 	g_type_class_add_private (klass, sizeof (GpmBrightnessKbdPrivate));
 }
@@ -572,14 +570,14 @@ adjust_kbd_brightness_according_to_ambient_light (GpmBrightnessKbd *brightness,
 		return FALSE;
 	}
 
-	gpm_light_sensor_get (brightness->priv->sensor, &ambient_light);
+	gpm_light_sensor_get_absolute (brightness->priv->sensor, &ambient_light);
 
 	/* this is also used if user reenables the keyboard backlight */
 	if (startup) {
 		state = STATE_FORCED_UNKNOWN;
 	}
 
- 	g_debug ("ambient light percent = %d", ambient_light);
+ 	gpm_debug ("ambient light percent = %d", ambient_light);
 
 	if (state == STATE_FORCED_UNKNOWN) {
 		/* if this is the first time we're launched with ambient light data... */
@@ -689,10 +687,11 @@ static void
 gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 {
 	gchar **names;
+	HalGManager *manager;
+	HalGDevice *device;
 
 	brightness->priv = GPM_BRIGHTNESS_KBD_GET_PRIVATE (brightness);
 
-	brightness->priv->hal = gpm_hal_new ();
 	brightness->priv->conf = gpm_conf_new ();
 
 	/* listen for ambient light changes.. if we have an ambient light sensor */
@@ -703,7 +702,9 @@ gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 	}
 
 	/* save udi of kbd adapter */
-	gpm_hal_device_find_capability (brightness->priv->hal, "keyboard_backlight", &names);
+	manager = hal_gmanager_new ();
+	hal_gmanager_find_capability (manager, "keyboard_backlight", &names, NULL);
+	g_object_unref (manager);
 	if (names == NULL || names[0] == NULL) {
 		gpm_warning ("No devices of capability keyboard_backlight");
 		return;
@@ -711,23 +712,26 @@ gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 
 	/* We only want first keyboard_backlight object (should only be one) */
 	brightness->priv->udi = g_strdup (names[0]);
-	gpm_hal_free_capability (brightness->priv->hal, names);
+	hal_gmanager_free_capability (names);
 
 	brightness->priv->does_own_dimming = FALSE;
 	brightness->priv->does_own_updates = FALSE;
 	brightness->priv->is_disabled = FALSE;
 
 	/* get a managed proxy */
-	brightness->priv->gproxy = gpm_proxy_new ();
-	gpm_proxy_assign (brightness->priv->gproxy,
-			  GPM_PROXY_SYSTEM,
+	brightness->priv->gproxy = dbus_proxy_new ();
+	dbus_proxy_assign (brightness->priv->gproxy,
+			  DBUS_PROXY_SYSTEM,
 			  HAL_DBUS_SERVICE,
 			  brightness->priv->udi,
 			  HAL_DBUS_INTERFACE_KBD_BACKLIGHT);
 
 	/* get levels that the adapter supports -- this does not change ever */
-	gpm_hal_device_get_uint (brightness->priv->hal, brightness->priv->udi, "keyboard_backlight.num_levels",
-				 &brightness->priv->levels);
+	device = hal_gdevice_new ();
+	hal_gdevice_set_udi (device, brightness->priv->udi);
+	hal_gdevice_get_uint (device, "keyboard_backlight.num_levels",
+			      &brightness->priv->levels, NULL);
+	g_object_unref (device);
 
 	/* this changes under our feet */
 	gpm_brightness_kbd_get_hw (brightness, &brightness->priv->current_hw);
@@ -753,21 +757,21 @@ gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 gboolean
 gpm_brightness_kbd_has_hw (void)
 {
-	GpmHal *hal;
+	HalGManager *manager;
 	gchar **names;
 	gboolean ret = TRUE;
 
 	/* okay, as singleton - so we don't allocate more memory */
-	hal = gpm_hal_new ();
-	gpm_hal_device_find_capability (hal, "keyboard_backlight", &names);
+	manager = hal_gmanager_new ();
+	hal_gmanager_find_capability (manager, "keyboard_backlight", &names, NULL);
+	g_object_unref (manager);
 
 	/* nothing found */
 	if (names == NULL || names[0] == NULL) {
 		ret = FALSE;
 	}
 
-	gpm_hal_free_capability (hal, names);
-	g_object_unref (hal);
+	hal_gmanager_free_capability (names);
 	return ret;
 }
 
