@@ -41,10 +41,7 @@ struct GpmGraphWidgetPrivate
 	gboolean		 use_grid;
 	gboolean		 use_legend;
 	gboolean		 use_events;
-
-	gboolean		 invert_x;
 	gboolean		 autorange_x;
-	gboolean		 invert_y;
 
 	GSList			*key_data; /* lines */
 	GSList			*key_event; /* dots */
@@ -101,7 +98,7 @@ gpm_graph_widget_key_find_id (GpmGraphWidget *graph, guint id)
 	GSList *ret;
 	ret = g_slist_find_custom (graph->priv->key_event, &id,
 				   gpm_graph_widget_key_compare_func);
-	if (! ret) {
+	if (ret == NULL) {
 		return NULL;
 	}
 	data = (GpmGraphWidgetKeyItem *) ret->data;
@@ -122,9 +119,14 @@ gpm_graph_widget_key_event_add (GpmGraphWidget *graph,
 	g_return_val_if_fail (graph != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
 
+	keyitem = gpm_graph_widget_key_find_id (graph, id);
+	if (keyitem != NULL) {
+		gpm_warning ("keyitem %i already in use", id);
+		return FALSE;
+	}
+
 	gpm_debug ("add to hashtable '%s'", desc);
 	keyitem = g_new0 (GpmGraphWidgetKeyItem, 1);
-
 	keyitem->id = id;
 	keyitem->desc = g_strdup (desc);
 	keyitem->colour = colour;
@@ -230,22 +232,6 @@ gpm_graph_widget_string_to_axis_type (const gchar *type)
 	return ret;
 }
 
-/**
- * gpm_graph_widget_set_title:
- * @graph: This class instance
- * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
- **/
-void
-gpm_graph_widget_set_title (GpmGraphWidget *graph, const gchar *title)
-{
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	if (graph->priv->title != NULL) {
-		g_free (graph->priv->title);
-	}
-	graph->priv->title = g_strdup (title);
-}
-
 /**
  * gpm_graph_widget_set_axis_type_x:
  * @graph: This class instance
@@ -332,19 +318,15 @@ gpm_graph_widget_init (GpmGraphWidget *graph)
 	PangoFontDescription *desc;
 
 	graph->priv = GPM_GRAPH_WIDGET_GET_PRIVATE (graph);
-	graph->priv->invert_x = FALSE;
-	graph->priv->invert_y = FALSE;
 	graph->priv->start_x = 0;
 	graph->priv->start_y = 0;
 	graph->priv->stop_x = 60;
 	graph->priv->stop_y = 100;
 	graph->priv->use_grid = TRUE;
 	graph->priv->use_legend = FALSE;
-	graph->priv->autorange_x = TRUE;
 	graph->priv->data_list = g_ptr_array_new ();
 	graph->priv->key_data = NULL;
 	graph->priv->key_event = NULL;
-	graph->priv->title = NULL;
 	graph->priv->axis_type_x = GPM_GRAPH_WIDGET_TYPE_TIME;
 	graph->priv->axis_type_y = GPM_GRAPH_WIDGET_TYPE_PERCENTAGE;
 
@@ -380,80 +362,80 @@ gpm_graph_widget_finalize (GObject *object)
 	context = pango_layout_get_context (graph->priv->layout);
 	g_object_unref (graph->priv->layout);
 	g_object_unref (context);
-
-	g_free (graph->priv->title);
 }
 
 /**
- * gpm_graph_widget_set_invert_x:
+ * gpm_graph_widget_data_add:
  * @graph: This class instance
- * @inv: If we should invert the axis
  *
- * Sets the inverse policy for the X axis, i.e. to count from 0..X or X..0
+ * Sets the data for the graph. You MUST NOT free the list before the widget.
  **/
-void
-gpm_graph_widget_set_invert_x (GpmGraphWidget *graph, gboolean inv)
+gboolean
+gpm_graph_widget_data_add (GpmGraphWidget *graph, GpmArray *array)
 {
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->invert_x = inv;
+	GpmArrayPoint *point;
+	guint length;
+	guint i;
+	guint oldx;
+
+	g_return_val_if_fail (array != NULL, FALSE);
+	g_return_val_if_fail (graph != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
+	g_return_val_if_fail (GPM_IS_ARRAY (array), FALSE);
+
+	/* check size is not zero */
+	length = gpm_array_get_size (array);
+	if (length == 0) {
+		gpm_warning ("Trying to assign a zero length array");
+		return FALSE;
+	}
+
+	/* check X is only monotomically increasing */
+	oldx = 0;
+	for (i=0; i < length; i++) {
+		point = gpm_array_get (array, i);
+		if (point->x < oldx) {
+			/* going backwards! */
+			return FALSE;
+		}
+		oldx = point->x;
+	}
+
+	/* always add, never remove in this function */
+	g_ptr_array_add (graph->priv->data_list, (gpointer) array);
+	return TRUE;
 }
 
 /**
- * gpm_graph_widget_set_invert_y:
+ * gpm_graph_widget_data_clear:
  * @graph: This class instance
- * @inv: If we should invert the axis
- *
- * Sets the inverse policy for the Y axis, i.e. to count from 0..Y or Y..0
- **/
-void
-gpm_graph_widget_set_invert_y (GpmGraphWidget *graph, gboolean inv)
-{
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->invert_y = inv;
-}
-
-/**
- * gpm_graph_widget_set_data:
- * @graph: This class instance
- * @list: The GList values to be plotted on the graph
  *
  * Sets the data for the graph. You MUST NOT free the list before the widget.
  **/
 void
-gpm_graph_widget_set_data (GpmGraphWidget *graph, GpmArray *array, guint id)
+gpm_graph_widget_data_clear (GpmGraphWidget *graph)
 {
-	g_return_if_fail (array != NULL);
-	g_return_if_fail (graph != NULL);
+	guint i;
+	guint length;
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
 
-	if (gpm_array_get_size (array) == 0) {
-		gpm_warning ("Trying to assign a zero length array");
-		return;
+	/* remove all in list */
+	length = graph->priv->data_list->len;
+	for (i=0; i < length; i++) {
+		gpm_debug ("Removing dataset %i", i);
+		g_ptr_array_remove_index_fast (graph->priv->data_list, 0);
 	}
-
-	/* fresh list */
-	if (graph->priv->data_list->len == 0 || graph->priv->data_list->len < id + 1) {
-		g_ptr_array_add (graph->priv->data_list, (gpointer) array);
-	} else {
-		/* remove existing, and add new */
-		gpm_debug ("Re-assigning dataset");
-		g_ptr_array_remove_index (graph->priv->data_list, id);
-		g_ptr_array_add (graph->priv->data_list, (gpointer) array);
-	}
-
 }
 
 /**
- * gpm_graph_widget_set_events:
+ * gpm_graph_widget_events_add:
  * @graph: This class instance
  * @list: The GList events to be plotted on the graph
  *
- * Sets the data for the graph. You MUST NOT free the list before the widget.
+ * Sets the data for the graph. You MUST NOT free the array before the widget.
  **/
 void
-gpm_graph_widget_set_events (GpmGraphWidget *graph, GpmArray *array)
+gpm_graph_widget_events_add (GpmGraphWidget *graph, GpmArray *array)
 {
 	g_return_if_fail (graph != NULL);
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
@@ -461,6 +443,20 @@ gpm_graph_widget_set_events (GpmGraphWidget *graph, GpmArray *array)
 	graph->priv->events = array;
 }
 
+/**
+ * gpm_graph_widget_events_clear:
+ * @graph: This class instance
+ *
+ * Clears the data for the graph.
+ **/
+void
+gpm_graph_widget_events_clear (GpmGraphWidget *graph)
+{
+	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
+	/* this is managed externally, so just set to NULL */
+	graph->priv->events = NULL;
+}
+
 /**
  * gpm_get_axis_label:
  * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
@@ -506,11 +502,11 @@ gpm_get_axis_label (GpmGraphWidgetAxisType axis, gint value)
 		/*Translators: This is %i Percentage*/
 		text = g_strdup_printf (_("%i%%"), value);
 	} else if (axis == GPM_GRAPH_WIDGET_TYPE_POWER) {
-		/*Translators: This is %i Watts*/
-		text = g_strdup_printf (_("%iW"), value / 1000);
+		/*Translators: This is %.1f Watts*/
+		text = g_strdup_printf (_("%.1fW"), (gfloat) value / 1000.0);
 	} else if (axis == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
-		/*Translators: This is %i Volts*/
-		text = g_strdup_printf (_("%iV"), value / 1000);
+		/*Translators: This is %.1f Volts*/
+		text = g_strdup_printf (_("%.1fV"), (gfloat) value / 1000.0);
 	} else {
 		text = g_strdup_printf ("%i", value);
 	}
@@ -584,11 +580,7 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 	cairo_set_source_rgb (cr, 0, 0, 0);
 	for (a=0; a<11; a++) {
 		b = graph->priv->box_x + (a * divwidth);
-		if (graph->priv->invert_x) {
-			value = (length_x / 10) * (10 - a) + graph->priv->start_x;
-		} else {
-			value = ((length_x / 10) * a) + graph->priv->start_x;
-		}
+		value = ((length_x / 10) * a) + graph->priv->start_x;
 		text = gpm_get_axis_label (graph->priv->axis_type_x, value);
 
 		pango_layout_set_text (graph->priv->layout, text, -1);
@@ -612,11 +604,7 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 	/* do y text */
 	for (a=0; a<11; a++) {
 		b = graph->priv->box_y + (a * divheight);
-		if (graph->priv->invert_y) {
-			value = (length_y / 10) * a + graph->priv->start_y;
-		} else {
-			value = (length_y / 10) * (10 - a) + graph->priv->start_y;
-		}
+		value = (length_y / 10) * (10 - a) + graph->priv->start_y;
 		text = gpm_get_axis_label (graph->priv->axis_type_y, value);
 
 		pango_layout_set_text (graph->priv->layout, text, -1);
@@ -655,6 +643,8 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 	gint biggest_y = 0;
 	gint smallest_x = 999999;
 	gint smallest_y = 999999;
+	guint rounding_x = 1;
+	guint rounding_y = 1;
 	GpmArray *array;
 	GpmArrayPoint *point;
 	guint i;
@@ -692,53 +682,68 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 	}
 	gpm_debug ("Data range is %i<x<%i, %i<y<%i", smallest_x, biggest_x, smallest_y, biggest_y);
 
-	/* do we autorange the start (so it starts at non-zero)? */
-	if (graph->priv->autorange_x) {
-		/* x is always time and always autoranges to the minute scale */
-		graph->priv->start_x = gpm_precision_round_down (smallest_x, 10*60);
-	} else {
-		graph->priv->start_x = 0;
-	}
-
-	/* x */
 	if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
-		graph->priv->stop_x = 100;
-	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
-		if (biggest_x > 10*60*60) {
-			graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 10*60*60) + smallest_x;
-		} else {
-			graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 10*60) + smallest_x;
-		}
+		rounding_x = 10;
 	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_POWER) {
-		graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 10000) + smallest_x;
+		rounding_x = 1000;
 	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
-		graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 1000) + smallest_x;
-	} else {
-		graph->priv->start_x = 0;
-		graph->priv->stop_x = biggest_x;
+		rounding_x = 1000;
+	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		if (biggest_x < 150) {
+			rounding_x = 150;
+		} else if (biggest_x < 5*60) {
+			rounding_x = 5 * 60;
+		} else {
+			rounding_x = 10 * 60;
+		}
+	}
+	if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+		rounding_y = 10;
+	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
+		rounding_y = 1000;
+	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+		rounding_y = 1000;
+	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		if (biggest_y < 150) {
+			rounding_y = 150;
+		} else if (biggest_y < 5*60) {
+			rounding_y = 5 * 60;
+		} else {
+			rounding_y = 10 * 60;
+		}
 	}
 
-	/* y */
-	if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
-		graph->priv->start_y = 0;
-		graph->priv->stop_y = 100;
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
-		graph->priv->start_y = 0;
-		if (biggest_y > 5*60*60) {
-			graph->priv->stop_y = gpm_precision_round_up (biggest_y, 5*60*60);
-		} else {
-			graph->priv->stop_y = gpm_precision_round_up (biggest_y, 5*60);
+	graph->priv->start_x = gpm_precision_round_down (smallest_x, rounding_x);
+	graph->priv->start_y = gpm_precision_round_down (smallest_y, rounding_y);
+	graph->priv->stop_x = gpm_precision_round_up (biggest_x, rounding_x);
+	graph->priv->stop_y = gpm_precision_round_up (biggest_y, rounding_y);
+
+	/* if percentage, and close to the end points, then extend */
+	if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+		if (graph->priv->stop_x >= 90) {
+			graph->priv->stop_x = 100;
+		}
+		if (graph->priv->start_x <= 10) {
+			graph->priv->start_x = 0;
+		}
+	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		if (graph->priv->start_x <= 60*10) {
+			graph->priv->start_x = 0;
 		}
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
-		graph->priv->start_y = 0;
-		graph->priv->stop_y = gpm_precision_round_up (biggest_y, 10000);
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
-		graph->priv->start_y = 0;
-		graph->priv->stop_y = gpm_precision_round_up (biggest_y, 1000);
-	} else {
-		graph->priv->start_y = 0;
-		graph->priv->stop_y = biggest_y;
 	}
+	if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+		if (graph->priv->stop_y >= 90) {
+			graph->priv->stop_y = 100;
+		}
+		if (graph->priv->start_y <= 10) {
+			graph->priv->start_y = 0;
+		}
+	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		if (graph->priv->start_y <= 60*10) {
+			graph->priv->start_y = 0;
+		}
+	}
+
 	gpm_debug ("Processed range is %i<x<%i, %i<y<%i",
 		   graph->priv->start_x, graph->priv->stop_x,
 		   graph->priv->start_y, graph->priv->stop_y);
@@ -956,9 +961,8 @@ gpm_graph_widget_draw_event_dots (GpmGraphWidget *graph, cairo_t *cr)
 	GpmArray *array = NULL;
 	GpmArrayPoint *point;
 	gint dot;
-	guint previous_point = 0;
 	gint prevpos = -1;
-
+	guint previous_y = 0;
 
 	if (graph->priv->events == NULL) {
 		/* we have no events */
@@ -991,11 +995,11 @@ gpm_graph_widget_draw_event_dots (GpmGraphWidget *graph, cairo_t *cr)
 
 		/* don't overlay the points, stack vertically */
 		if (abs (newx - prevpos) < 10) {
-			previous_point++;
-		} else {
-			previous_point = 0;
+			newy = previous_y - 8;
 		}
-		newy -= (8 * previous_point);
+
+		/* save the last y point */
+		previous_y = newy;
 
 		/* only do the event dot, if it's going to be valid on the graph */
 		if (point->x > graph->priv->start_x && newy > graph->priv->box_y) {
@@ -1143,6 +1147,11 @@ gpm_graph_widget_legend_calculate_size (GpmGraphWidget *graph, cairo_t *cr,
 		}
 	}
 
+	/* have we got no entries? */
+	if (*width == 0 && *height == 0) {
+		return TRUE;
+	}
+
 	/* add for borders */
 	*width += 25;
 	*height += 3;
@@ -1166,8 +1175,6 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	guint legend_width = 0;
 	gint data_x;
 	gint data_y;
-	PangoRectangle ink_rect_title;
-	PangoRectangle logical_rect;
 
 	GpmGraphWidget *graph = (GpmGraphWidget*) graph_widget;
 	g_return_if_fail (graph != NULL);
@@ -1180,16 +1187,10 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	graph->priv->box_x = 45;
 	graph->priv->box_y = 5;
 
-	if (graph->priv->title != NULL) {
-		pango_layout_set_text (graph->priv->layout, graph->priv->title, -1);
-		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect_title, &logical_rect);
-		graph->priv->box_y += ink_rect_title.height;
-	}
-
 	graph->priv->box_height = graph_widget->allocation.height - (20 + graph->priv->box_y);
 
 	/* make size adjustment for legend */
-	if (graph->priv->use_legend) {
+	if (graph->priv->use_legend && legend_height > 0) {
 		graph->priv->box_width = graph_widget->allocation.width -
 					 (3 + legend_width + 5 + graph->priv->box_x);
 		legend_x = graph->priv->box_x + graph->priv->box_width + 6;
@@ -1199,13 +1200,6 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 					 (3 + graph->priv->box_x);
 	}
 
-	/* center title */
-	if (graph->priv->title != NULL) {
-		cairo_move_to (cr, graph->priv->box_x + (graph->priv->box_width-ink_rect_title.width)/2, 9);
-		pango_layout_set_text(graph->priv->layout, graph->priv->title, -1);
-		pango_cairo_show_layout(cr, graph->priv->layout);
-	}
-
 	/* graph background */
 	gpm_graph_widget_draw_bounding_box (cr, graph->priv->box_x, graph->priv->box_y,
 				     graph->priv->box_width, graph->priv->box_height);
@@ -1225,11 +1219,11 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	gpm_graph_widget_draw_labels (graph, cr);
 	gpm_graph_widget_draw_line (graph, cr);
 
-	if (graph->priv->use_events) {
+	if (graph->priv->use_events == TRUE) {
 		gpm_graph_widget_draw_event_dots (graph, cr);
 	}
 
-	if (graph->priv->use_legend) {
+	if (graph->priv->use_legend == TRUE && legend_height > 0) {
 		gpm_graph_widget_draw_legend (graph, legend_x, legend_y, legend_width, legend_height);
 	}
 
@@ -1271,3 +1265,298 @@ gpm_graph_widget_new (void)
 {
 	return g_object_new (GPM_TYPE_GRAPH_WIDGET, NULL);
 }
+
+/***************************************************************************
+ ***                          MAKE CHECK TESTS                           ***
+ ***************************************************************************/
+#ifdef GPM_BUILD_TESTS
+#include "gpm-self-test.h"
+
+GtkWidget *window;
+GtkWidget *graph;
+GtkWidget *label;
+
+static gint
+close_handler (GtkWidget *widget, gpointer gdata)
+{
+	gtk_main_quit ();
+	return FALSE;
+}
+
+static void
+clicked_passed_cb (GtkWidget *widget, gpointer gdata)
+{
+	GpmSelfTest *test = (GpmSelfTest *) gdata;
+	gpm_st_success (test, NULL);
+	gtk_main_quit ();
+}
+
+static void
+clicked_failed_cb (GtkWidget *widget, gpointer gdata)
+{
+	GpmSelfTest *test = (GpmSelfTest *) gdata;
+	gpm_st_failed (test, NULL);
+	gtk_main_quit ();
+}
+
+static void
+create_graph_window (GpmSelfTest *test)
+{
+	GtkWidget *button_passed;
+	GtkWidget *button_failed;
+	GtkWidget *vbox;
+
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	graph = gpm_graph_widget_new ();
+	vbox = gtk_vbox_new (FALSE, 0);
+	label = gtk_label_new("Title");
+
+	button_passed = gtk_button_new_with_label("Passed");
+	gtk_signal_connect(GTK_OBJECT(button_passed), "clicked", GTK_SIGNAL_FUNC(clicked_passed_cb), test);
+	button_failed = gtk_button_new_with_label("Failed");
+	gtk_signal_connect(GTK_OBJECT(button_failed), "clicked", GTK_SIGNAL_FUNC(clicked_failed_cb), test);
+
+	gtk_widget_set_size_request (graph, 600, 300);
+	gtk_signal_connect (GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(close_handler), test);
+
+	gtk_container_add (GTK_CONTAINER (window), vbox);
+	gtk_box_pack_start (GTK_BOX (vbox), graph, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), button_passed, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), button_failed, FALSE, FALSE, 0);
+	gtk_container_border_width (GTK_CONTAINER (window), 0);
+
+	gtk_widget_show (vbox);
+	gtk_widget_show (label);
+	gtk_widget_show (button_passed);
+	gtk_widget_show (button_failed);
+}
+
+static void
+wait_for_input (GpmSelfTest *test)
+{
+	gtk_widget_hide_all (window);
+	gtk_widget_show_all (window);
+	gtk_main ();
+}
+
+void
+gpm_st_title_graph (GpmSelfTest *test, const gchar *format, ...)
+{
+	va_list args;
+	gchar va_args_buffer [1025];
+	va_start (args, format);
+	g_vsnprintf (va_args_buffer, 1024, format, args);
+	va_end (args);
+	gpm_st_title (test, va_args_buffer);
+	gtk_label_set_label (GTK_LABEL (label), va_args_buffer);
+//	g_print ("> check #%u\t%s: \t%s...", test->total+1, test->type, va_args_buffer);
+//	test->total++;
+}
+
+void
+gpm_st_graph_widget (GpmSelfTest *test)
+{
+	GpmArray *data;
+	GpmArray *data_more;
+	GpmArray *events;
+	gboolean ret;
+
+	if (gpm_st_start (test, "GpmGraphWidget", CLASS_MANUAL) == FALSE) {
+		return;
+	}
+
+	create_graph_window (test);
+	gpm_graph_widget_enable_legend (GPM_GRAPH_WIDGET (graph), TRUE);
+	gpm_graph_widget_enable_events (GPM_GRAPH_WIDGET (graph), TRUE);
+
+	/********** TYPES *************/
+	gpm_st_title_graph (test, "graph loaded, visible, no key, and set to y=percent x=time");
+	wait_for_input (test);
+
+	gpm_graph_widget_set_axis_type_x (GPM_GRAPH_WIDGET (graph), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
+	gpm_graph_widget_set_axis_type_y (GPM_GRAPH_WIDGET (graph), GPM_GRAPH_WIDGET_TYPE_TIME);
+
+	gpm_st_title_graph (test, "now set to y=time x=percent");
+	wait_for_input (test);
+
+	/********** KEY DATA *************/
+	gpm_graph_widget_key_data_add (GPM_GRAPH_WIDGET (graph), GPM_COLOUR_RED, "red data");
+	gpm_graph_widget_key_data_add (GPM_GRAPH_WIDGET (graph), GPM_COLOUR_GREEN, "green data");
+	gpm_graph_widget_key_data_add (GPM_GRAPH_WIDGET (graph), GPM_COLOUR_BLUE, "blue data");
+
+	gpm_st_title_graph (test, "red green blue key data added");
+	wait_for_input (test);
+
+	gpm_graph_widget_key_data_clear (GPM_GRAPH_WIDGET (graph));
+
+	gpm_st_title_graph (test, "data items cleared, no key remains");
+	wait_for_input (test);
+
+	/********** KEY EVENT *************/
+	gpm_graph_widget_key_event_add (GPM_GRAPH_WIDGET (graph), 0,
+					GPM_COLOUR_RED,
+					GPM_GRAPH_WIDGET_SHAPE_CIRCLE,
+					"red circle");
+	gpm_graph_widget_key_event_add (GPM_GRAPH_WIDGET (graph), 1,
+					GPM_COLOUR_GREEN,
+					GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+					"green square");
+	gpm_graph_widget_key_event_add (GPM_GRAPH_WIDGET (graph), 2,
+					GPM_COLOUR_BLUE,
+					GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+					"blue triangle");
+	gpm_graph_widget_key_event_add (GPM_GRAPH_WIDGET (graph), 3,
+					GPM_COLOUR_WHITE,
+					GPM_GRAPH_WIDGET_SHAPE_DIAMOND,
+					"white diamond");
+
+	gpm_st_title_graph (test, "red green blue white key events added");
+	wait_for_input (test);
+
+
+	/********** KEY EVENT duplicate test *************/
+	gpm_st_title (test, "duplicate key event test");
+	ret = gpm_graph_widget_key_event_add (GPM_GRAPH_WIDGET (graph), 3,
+					      GPM_COLOUR_WHITE,
+					      GPM_GRAPH_WIDGET_SHAPE_DIAMOND,
+					      "white diamond");
+	if (ret == FALSE) {
+		gpm_st_success (test, "refused duplicate id");
+	} else {
+		gpm_st_failed (test, "added duplicate ID!");
+	}
+
+	gpm_graph_widget_key_event_clear (GPM_GRAPH_WIDGET (graph));
+
+	gpm_st_title_graph (test, "event items cleared, no key remains");
+	wait_for_input (test);
+
+	/********** DATA *************/
+	gpm_graph_widget_set_axis_type_x (GPM_GRAPH_WIDGET (graph), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
+	gpm_graph_widget_set_axis_type_y (GPM_GRAPH_WIDGET (graph), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
+
+	gpm_graph_widget_key_data_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_graph_widget_key_event_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_graph_widget_key_data_add (GPM_GRAPH_WIDGET (graph), GPM_COLOUR_RED, "red data");
+	gpm_graph_widget_key_data_add (GPM_GRAPH_WIDGET (graph), GPM_COLOUR_BLUE, "blue data");
+	gpm_graph_widget_key_event_add (GPM_GRAPH_WIDGET (graph), 0, GPM_COLOUR_GREEN, GPM_GRAPH_WIDGET_SHAPE_SQUARE, "green square");
+	
+	/********** ADD INVALID DATA *************/
+	data = gpm_array_new ();
+	gpm_array_append (data, 50, 0, GPM_COLOUR_RED);
+	gpm_array_append (data, 40, 100, GPM_COLOUR_RED);
+	gpm_graph_widget_data_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_st_title (test, "add invalid data");
+	ret = gpm_graph_widget_data_add (GPM_GRAPH_WIDGET (graph), data);
+	if (ret == FALSE) {
+		gpm_st_success (test, "ignored");
+	} else {
+		gpm_st_failed (test, "failed to ignore invalid data");
+	}
+	g_object_unref (data);
+
+	/********** ADD NO DATA *************/
+	data = gpm_array_new ();
+	gpm_st_title (test, "add zero data");
+	gpm_graph_widget_data_clear (GPM_GRAPH_WIDGET (graph));
+	ret = gpm_graph_widget_data_add (GPM_GRAPH_WIDGET (graph), data);
+	if (ret == FALSE) {
+		gpm_st_success (test, "ignored");
+	} else {
+		gpm_st_failed (test, "failed to ignore zero data");
+	}
+	g_object_unref (data);
+
+	/********** ADD VALID DATA *************/
+	data = gpm_array_new ();
+	gpm_array_append (data, 0, 0, GPM_COLOUR_RED);
+	gpm_array_append (data, 100, 100, GPM_COLOUR_RED);
+	gpm_st_title (test, "add valid data");
+	ret = gpm_graph_widget_data_add (GPM_GRAPH_WIDGET (graph), data);
+	if (ret == TRUE) {
+		gpm_st_success (test, NULL);
+	} else {
+		gpm_st_failed (test, "failed to add valid data");
+	}
+
+	/********** SHOW VALID DATA *************/
+	gpm_st_title_graph (test, "red line shown gradient up");
+	wait_for_input (test);
+
+	/*********** second line **************/
+	data_more = gpm_array_new ();
+	gpm_array_append (data_more, 0, 100, GPM_COLOUR_BLUE);
+	gpm_array_append (data_more, 100, 0, GPM_COLOUR_BLUE);
+	gpm_graph_widget_data_add (GPM_GRAPH_WIDGET (graph), data_more);
+
+	gpm_st_title_graph (test, "red line shown gradient up, blue gradient down");
+	wait_for_input (test);
+
+	/*********** dots **************/
+	events = gpm_array_new ();
+	gpm_array_append (events, 25, 0, 0);
+	gpm_array_append (events, 50, 0, 0);
+	gpm_array_append (events, 75, 0, 0);
+	gpm_graph_widget_events_add (GPM_GRAPH_WIDGET (graph), events);
+
+	gpm_st_title_graph (test, "events follow red line (primary)");
+	wait_for_input (test);
+
+	/*********** stacked dots **************/
+	gpm_array_append (events, 76, 0, 0);
+	gpm_array_append (events, 77, 0, 0);
+	gpm_graph_widget_events_add (GPM_GRAPH_WIDGET (graph), events);
+
+	gpm_st_title_graph (test, "three events stacked at ~75");
+	wait_for_input (test);
+
+	/*********** events removed **************/
+	gpm_graph_widget_events_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_st_title_graph (test, "events removed");
+	wait_for_input (test);
+
+	/*********** data lines removed **************/
+	gpm_graph_widget_data_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_st_title_graph (test, "all lines and event removed");
+	wait_for_input (test);
+
+	g_object_unref (events);
+	g_object_unref (data);
+	g_object_unref (data_more);
+
+	/********** AUTORANGING PERCENT (close) *************/
+	gpm_graph_widget_set_axis_type_x (GPM_GRAPH_WIDGET (graph), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
+	gpm_graph_widget_key_event_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_graph_widget_key_data_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_graph_widget_key_data_add (GPM_GRAPH_WIDGET (graph), GPM_COLOUR_RED, "red data");
+	data = gpm_array_new ();
+	gpm_array_append (data, 0, 75, GPM_COLOUR_RED);
+	gpm_array_append (data, 20, 78, GPM_COLOUR_RED);
+	gpm_array_append (data, 40, 74, GPM_COLOUR_RED);
+	gpm_array_append (data, 60, 72, GPM_COLOUR_RED);
+	gpm_array_append (data, 80, 78, GPM_COLOUR_RED);
+	gpm_array_append (data, 100, 79, GPM_COLOUR_RED);
+	gpm_graph_widget_data_add (GPM_GRAPH_WIDGET (graph), data);
+	gpm_st_title_graph (test, "autorange y axis between 70 and 80");
+	wait_for_input (test);
+	g_object_unref (data);
+
+	/********** AUTORANGING PERCENT (extremes) *************/
+	data = gpm_array_new ();
+	gpm_array_append (data, 0, 6, GPM_COLOUR_RED);
+	gpm_array_append (data, 100, 85, GPM_COLOUR_RED);
+	gpm_graph_widget_data_clear (GPM_GRAPH_WIDGET (graph));
+	gpm_graph_widget_data_add (GPM_GRAPH_WIDGET (graph), data);
+	gpm_st_title_graph (test, "autorange y axis between 0 and 100");
+	wait_for_input (test);
+	g_object_unref (data);
+
+	/* hide window */
+	gtk_widget_hide_all (window);
+
+	gpm_st_end (test);
+}
+
+#endif
+
