@@ -467,3 +467,224 @@ gpm_inhibit_new (void)
 	inhibit = g_object_new (GPM_TYPE_INHIBIT, NULL);
 	return GPM_INHIBIT (inhibit);
 }
+
+/***************************************************************************
+ ***                          MAKE CHECK TESTS                           ***
+ ***************************************************************************/
+#ifdef GPM_BUILD_TESTS
+#include "gpm-self-test.h"
+#include <libdbus-proxy.h>
+#include "gpm-common.h"
+
+/** cookie is returned as an unsigned integer */
+static gboolean
+inhibit (DbusProxy       *gproxy,
+	 const gchar     *appname,
+	 const gchar     *reason,
+	 guint           *cookie)
+{
+	GError  *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	g_return_val_if_fail (cookie != NULL, FALSE);
+
+	proxy = dbus_proxy_get_proxy (gproxy);
+	if (proxy == NULL) {
+		g_warning ("not connected");
+		return FALSE;
+	}
+
+	ret = dbus_g_proxy_call (proxy, "Inhibit", &error,
+				 G_TYPE_STRING, appname,
+				 G_TYPE_STRING, reason,
+				 G_TYPE_INVALID,
+				 G_TYPE_UINT, cookie,
+				 G_TYPE_INVALID);
+	if (error) {
+		g_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+		*cookie = 0;
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		g_warning ("Inhibit failed!");
+	}
+
+	return ret;
+}
+
+static gboolean
+uninhibit (DbusProxy *gproxy,
+	   guint      cookie)
+{
+	GError *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	proxy = dbus_proxy_get_proxy (gproxy);
+	if (proxy == NULL) {
+		g_warning ("not connected");
+		return FALSE;
+	}
+
+	ret = dbus_g_proxy_call (proxy, "UnInhibit", &error,
+				 G_TYPE_UINT, cookie,
+				 G_TYPE_INVALID,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	return ret;
+}
+
+static gboolean
+has_inhibit (DbusProxy *gproxy,
+			      gboolean        *has_inhibit)
+{
+	GError  *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	proxy = dbus_proxy_get_proxy (gproxy);
+	if (proxy == NULL) {
+		g_warning ("not connected");
+		return FALSE;
+	}
+
+	ret = dbus_g_proxy_call (proxy, "HasInhibit", &error,
+				 G_TYPE_INVALID,
+				 G_TYPE_BOOLEAN, has_inhibit,
+				 G_TYPE_INVALID);
+	if (error) {
+		g_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		g_warning ("HasInhibit failed!");
+	}
+
+	return ret;
+}
+
+void
+gpm_st_inhibit (GpmSelfTest *test)
+{
+	gboolean ret;
+	gboolean valid;
+	guint cookie1 = 0;
+	guint cookie2 = 0;
+	DbusProxy *gproxy;
+
+	if (gpm_st_start (test, "GpmInhibit", CLASS_AUTO) == FALSE) {
+		return;
+	}
+
+	gproxy = dbus_proxy_new ();
+	dbus_proxy_assign (gproxy, DBUS_PROXY_SESSION,
+				   GPM_DBUS_SERVICE,
+				   GPM_DBUS_PATH_INHIBIT,
+				   GPM_DBUS_INTERFACE_INHIBIT);
+
+	if (gproxy == NULL) {
+		g_warning ("Unable to get connection to power manager");
+		return;
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we are not inhibited");
+	ret = has_inhibit (gproxy, &valid);
+	if (ret == FALSE) {
+		gpm_st_failed (test, "Unable to test validity");
+	} else if (valid == TRUE) {
+		gpm_st_failed (test, "Already inhibited");
+	} else {
+		gpm_st_success (test, NULL);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "clear an invalid cookie");
+	ret = uninhibit (gproxy, 123456);
+	if (ret == FALSE) {
+		gpm_st_success (test, "invalid cookie failed as expected");
+	} else {
+		gpm_st_failed (test, "should have rejected invalid cookie");
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "get auto cookie 1");
+	ret = inhibit (gproxy,
+				  "gnome-power-self-test",
+				  "test inhibit",
+				  &cookie1);
+	if (ret == FALSE) {
+		gpm_st_failed (test, "Unable to inhibit");
+	} else if (cookie1 == 0) {
+		gpm_st_failed (test, "Cookie invalid (cookie: %u)", cookie1);
+	} else {
+		gpm_st_success (test, "cookie: %u", cookie1);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we are auto inhibited");
+	ret = has_inhibit (gproxy, &valid);
+	if (ret == FALSE) {
+		gpm_st_failed (test, "Unable to test validity");
+	} else if (valid == TRUE) {
+		gpm_st_success (test, "inhibited");
+	} else {
+		gpm_st_failed (test, "inhibit failed");
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "get cookie 2");
+	ret = inhibit (gproxy,
+				  "gnome-power-self-test",
+				  "test inhibit",
+				  &cookie2);
+	if (ret == FALSE) {
+		gpm_st_failed (test, "Unable to inhibit");
+	} else if (cookie2 == 0) {
+		gpm_st_failed (test, "Cookie invalid (cookie: %u)", cookie2);
+	} else {
+		gpm_st_success (test, "cookie: %u", cookie2);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "clear cookie 1");
+	ret = uninhibit (gproxy, cookie1);
+	if (ret == FALSE) {
+		gpm_st_failed (test, "cookie failed to clear");
+	} else {
+		gpm_st_success (test, NULL);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we are still inhibited");
+	ret = has_inhibit (gproxy, &valid);
+	if (ret == FALSE) {
+		gpm_st_failed (test, "Unable to test validity");
+	} else if (valid == TRUE) {
+		gpm_st_success (test, "inhibited");
+	} else {
+		gpm_st_failed (test, "inhibit failed");
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "clear cookie 2");
+	ret = uninhibit (gproxy, cookie2);
+	if (ret == FALSE) {
+		gpm_st_failed (test, "cookie failed to clear");
+	} else {
+		gpm_st_success (test, NULL);
+	}
+
+	g_object_unref (gproxy);
+
+	gpm_st_end (test);
+}
+
+#endif
+
