@@ -32,17 +32,18 @@
 #include <math.h>
 #include <string.h>
 
+#include <libhal-gmanager.h>
+#include <libhal-gcpufreq.h>
+
 #include "gpm-tray-icon.h"
 #include "gpm-common.h"
 #include "gpm-prefs.h"
 #include "gpm-conf.h"
-#include "gpm-hal.h"
-#include "gpm-cpufreq.h"
 #include "gpm-prefs-core.h"
 #include "gpm-debug.h"
 #include "gpm-stock-icons.h"
 #include "gpm-screensaver.h"
-#include "gpm-powermanager.h"
+#include "gpm-prefs-server.h"
 
 static void     gpm_prefs_class_init (GpmPrefsClass *klass);
 static void     gpm_prefs_init       (GpmPrefs      *prefs);
@@ -56,15 +57,15 @@ struct GpmPrefsPrivate
 	gboolean		 has_batteries;
 	gboolean		 has_lcd;
 	gboolean		 has_ups;
+	gboolean		 has_ambient;
 	gboolean		 has_button_lid;
 	gboolean		 has_button_suspend;
 	gboolean		 can_suspend;
 	gboolean		 can_hibernate;
 	GpmConf			*conf;
 	GpmScreensaver		*screensaver;
-	GpmCpuFreq		*cpufreq;
-	GpmCpuFreqEnum		 cpufreq_types;
-	GpmHal			*hal;
+	HalGCpufreq		*hal_cpufreq;
+	HalGCpufreqType		 cpufreq_types;
 };
 
 enum {
@@ -165,8 +166,8 @@ gpm_dbus_method_bool (const gchar *method)
 
 	proxy = dbus_g_proxy_new_for_name (connection,
 					   GPM_DBUS_SERVICE,
-					   GPM_DBUS_PATH_CONTROL,
-					   GPM_DBUS_INTERFACE_CONTROL);
+					   GPM_DBUS_PATH,
+					   GPM_DBUS_INTERFACE);
 	ret = dbus_g_proxy_call (proxy, method, &error,
 				 G_TYPE_INVALID,
 				 G_TYPE_BOOLEAN, &value,
@@ -184,6 +185,50 @@ gpm_dbus_method_bool (const gchar *method)
 	return value;
 }
 
+/**
+ * gpm_dbus_method_int:
+ * @method: The g-p-m DBUS method name, e.g. "AllowedSuspend"
+ **/
+static gint
+gpm_dbus_method_int (const gchar *method)
+{
+	DBusGConnection *connection;
+	DBusGProxy *proxy;
+	GError *error;
+	gboolean ret;
+	gint value = 0;
+	error = NULL;
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+	if (connection == NULL) {
+		if (error) {
+			gpm_warning ("Couldn't connect to PowerManager %s",
+				     error->message);
+			g_error_free (error);
+		}
+		return FALSE;
+	}
+
+	proxy = dbus_g_proxy_new_for_name (connection,
+					   GPM_DBUS_SERVICE,
+					   GPM_DBUS_PATH,
+					   GPM_DBUS_INTERFACE);
+	ret = dbus_g_proxy_call (proxy, method, &error,
+				 G_TYPE_INVALID,
+				 G_TYPE_INT, &value,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("%s failed!", method);
+		return FALSE;
+	}
+	g_object_unref (proxy);
+	return value;
+}
+
 /**
  * gpm_prefs_help_cb:
  * @widget: The GtkWidget object
@@ -210,8 +255,8 @@ gpm_prefs_icon_radio_cb (GtkWidget *widget,
 
 	policy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "policy"));
 	str = gpm_tray_icon_mode_to_string (policy);
-	gpm_debug ("Changing %s to %s", GPM_CONF_ICON_POLICY, str);
-	gpm_conf_set_string (prefs->priv->conf, GPM_CONF_ICON_POLICY, str);
+	gpm_debug ("Changing %s to %s", GPM_CONF_UI_ICON_POLICY, str);
+	gpm_conf_set_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, str);
 }
 
 /**
@@ -484,8 +529,7 @@ gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
 						   ACTION_NOTHING_TEXT);
 			n_added++;
 		} else {
-			gpm_critical_error ("Unknown action read from conf: %s",
-					    actions[i]);
+			gpm_error ("Unknown action read from conf: %s", actions[i]);
 		}
 
 		if (strcmp (value, actions[i]) == 0)
@@ -630,16 +674,16 @@ conf_key_changed_cb (GpmConf     *conf,
 {
 	gboolean  enabled;
 
-	if (strcmp (key, GPM_CONF_AC_LOWPOWER) == 0) {
-		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_AC_LOWPOWER, &enabled);
+	if (strcmp (key, GPM_CONF_LOWPOWER_AC) == 0) {
+		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_LOWPOWER_AC, &enabled);
 		gpm_debug ("need to enable checkbox");
 
-	} else if (strcmp (key, GPM_CONF_UPS_LOWPOWER) == 0) {
-		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_UPS_LOWPOWER, &enabled);
+	} else if (strcmp (key, GPM_CONF_LOWPOWER_UPS) == 0) {
+		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_LOWPOWER_UPS, &enabled);
 		gpm_debug ("need to enable checkbox");
 
-	} else if (strcmp (key, GPM_CONF_BATTERY_LOWPOWER) == 0) {
-		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_BATTERY_LOWPOWER, &enabled);
+	} else if (strcmp (key, GPM_CONF_LOWPOWER_BATT) == 0) {
+		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_LOWPOWER_BATT, &enabled);
 		gpm_debug ("need to enable checkbox");
 	}
 }
@@ -693,14 +737,14 @@ static void
 gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 				 const gchar      *widget_name,
 				 const gchar      *gpm_pref_key,
-				 GpmCpuFreqEnum cpufreq_types)
+				 HalGCpufreqType cpufreq_types)
 {
 	gchar *value;
 	guint n_added = 0;
 	gboolean has_option = FALSE;
 	gboolean is_writable;
 	GtkWidget *widget;
-	GpmCpuFreqEnum cpufreq_type;
+	HalGCpufreqType cpufreq_type;
 
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
 	gpm_conf_get_string (prefs->priv->conf, gpm_pref_key, &value);
@@ -718,55 +762,55 @@ gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 			  G_CALLBACK (gpm_prefs_processor_combo_changed_cb),
 			  prefs);
 
-	cpufreq_type = gpm_cpufreq_string_to_enum (value);
+	cpufreq_type = hal_gcpufreq_string_to_enum (value);
 
-	if (cpufreq_types & GPM_CPUFREQ_ONDEMAND) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_ONDEMAND) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_ONDEMAND_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_ONDEMAND) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_ONDEMAND) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_NOTHING) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_NOTHING) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_NOTHING_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_ONDEMAND) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_ONDEMAND) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_CONSERVATIVE) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_CONSERVATIVE) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_CONSERVATIVE_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_CONSERVATIVE) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_CONSERVATIVE) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_POWERSAVE) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_POWERSAVE) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_POWERSAVE_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_POWERSAVE) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_POWERSAVE) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_PERFORMANCE) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_PERFORMANCE) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_PERFORMANCE_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_PERFORMANCE) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_PERFORMANCE) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
 
-	if (has_option == FALSE || cpufreq_type == GPM_CPUFREQ_NOTHING) {
+	if (has_option == FALSE || cpufreq_type == LIBHAL_CPUFREQ_NOTHING) {
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 	}
 	g_free (value);
@@ -785,7 +829,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	GtkWidget   *radiobutton_icon_never;
 	gboolean     is_writable;
 
-	gpm_conf_get_string (prefs->priv->conf, GPM_CONF_ICON_POLICY, &icon_policy_str);
+	gpm_conf_get_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, &icon_policy_str);
 	icon_policy = gpm_tray_icon_mode_from_string (icon_policy_str);
 	g_free (icon_policy_str);
 
@@ -800,7 +844,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	radiobutton_icon_never = glade_xml_get_widget (prefs->priv->glade_xml,
 							"radiobutton_notification_never");
 
-	gpm_conf_is_writable (prefs->priv->conf, GPM_CONF_ICON_POLICY, &is_writable);
+	gpm_conf_is_writable (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, &is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_always, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_present, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_charge, is_writable);
@@ -844,7 +888,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 
 	/* set up the sound checkbox */
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_notification_sound",
-	  			  GPM_CONF_ENABLE_BEEPING);
+	  			  GPM_CONF_UI_ENABLE_BEEPING);
 
 	if (prefs->priv->has_batteries == TRUE) {
 		/* there's no use case for displaying this option */
@@ -875,16 +919,19 @@ prefs_setup_ac (GpmPrefs *prefs)
 				 NULL};
 
 	gpm_prefs_setup_action_combo (prefs, "combobox_ac_lid",
-				      GPM_CONF_AC_BUTTON_LID,
+				      GPM_CONF_BUTTON_LID_AC,
 				      button_lid_actions);
 	gpm_prefs_setup_processor_combo (prefs, "combobox_ac_cpu",
-					 GPM_CONF_AC_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+					 GPM_CONF_CPUFREQ_POLICY_AC, prefs->priv->cpufreq_types);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_computer",
-				      GPM_CONF_AC_SLEEP_COMPUTER);
+				      GPM_CONF_TIMEOUT_SLEEP_COMPUTER_AC);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_display",
-				      GPM_CONF_AC_SLEEP_DISPLAY);
+				      GPM_CONF_TIMEOUT_SLEEP_DISPLAY_AC);
 	gpm_prefs_setup_brightness_slider (prefs, "hscale_ac_brightness",
-					   GPM_CONF_AC_BRIGHTNESS);
+					   GPM_CONF_BACKLIGHT_BRIGHTNESS_AC);
+
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_ac_display_dim",
+				  GPM_CONF_BACKLIGHT_IDLE_DIM_AC);
 
 	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
 	set_idle_hscale_stops (prefs, "hscale_ac_computer", delay);
@@ -894,16 +941,18 @@ prefs_setup_ac (GpmPrefs *prefs)
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_lid");
 		gtk_widget_hide_all (widget);
 	}
-	if (prefs->priv->cpufreq == NULL) {
+	if (prefs->priv->hal_cpufreq == NULL) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_cpu");
 		gtk_widget_hide_all (widget);
 	}
 	if (prefs->priv->has_lcd == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_brightness");
 		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_ac_display_dim");
+		gtk_widget_hide_all (widget);
 	}
 
-	gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_SHOW_CPUFREQ_UI, &show_cpufreq);
+	gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_UI_SHOW_CPUFREQ, &show_cpufreq);
 	if (show_cpufreq == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_cpu");
 		gtk_widget_hide_all (widget);
@@ -918,7 +967,7 @@ prefs_setup_battery (GpmPrefs *prefs)
 	gint delay;
 	gint page;
 	gboolean show_cpufreq;
-	
+
 	const gchar  *button_lid_actions[] =
 				{ACTION_NOTHING,
 				 ACTION_BLANK,
@@ -941,19 +990,27 @@ prefs_setup_battery (GpmPrefs *prefs)
 	}
 
 	gpm_prefs_setup_action_combo (prefs, "combobox_battery_lid",
-				      GPM_CONF_BATTERY_BUTTON_LID,
+				      GPM_CONF_BUTTON_LID_BATT,
 				      button_lid_actions);
 	gpm_prefs_setup_action_combo (prefs, "combobox_battery_critical",
-				      GPM_CONF_BATTERY_CRITICAL,
+				      GPM_CONF_ACTIONS_CRITICAL_BATT,
 				      battery_critical_actions);
 	gpm_prefs_setup_processor_combo (prefs, "combobox_battery_cpu",
-					 GPM_CONF_BATTERY_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+					 GPM_CONF_CPUFREQ_POLICY_BATT, prefs->priv->cpufreq_types);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_computer",
-				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
+				      GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_display",
-				      GPM_CONF_BATTERY_SLEEP_DISPLAY);
+				      GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT);
 	gpm_prefs_setup_brightness_slider (prefs, "hscale_battery_brightness",
-					   GPM_CONF_BATTERY_BRIGHTNESS);
+					   GPM_CONF_BACKLIGHT_BRIGHTNESS_BATT);
+
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_battery_display_dim",
+				  GPM_CONF_BACKLIGHT_IDLE_DIM_BATT);
+
+	if (prefs->priv->has_ambient == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_general_ambient");
+		gtk_widget_hide_all (widget);
+	}
 
 	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
 	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
@@ -963,15 +1020,17 @@ prefs_setup_battery (GpmPrefs *prefs)
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_lid");
 		gtk_widget_hide_all (widget);
 	}
-	if (prefs->priv->cpufreq == NULL) {
+	if (prefs->priv->hal_cpufreq == NULL) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_cpu");
 		gtk_widget_hide_all (widget);
 	}
 	if (prefs->priv->has_lcd == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_brightness");
 		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_battery_display_dim");
+		gtk_widget_hide_all (widget);
 	}
-	gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_SHOW_CPUFREQ_UI, &show_cpufreq);
+	gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_UI_SHOW_CPUFREQ, &show_cpufreq);
 	if (show_cpufreq == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_cpu");
 		gtk_widget_hide_all (widget);
@@ -1001,13 +1060,13 @@ prefs_setup_ups (GpmPrefs *prefs)
 	}
 
 	gpm_prefs_setup_action_combo (prefs, "combobox_ups_low",
-				      GPM_CONF_UPS_LOW,
+				      GPM_CONF_ACTIONS_LOW_UPS,
 				      ups_low_actions);
 	gpm_prefs_setup_action_combo (prefs, "combobox_ups_critical",
-				      GPM_CONF_UPS_CRITICAL,
+				      GPM_CONF_ACTIONS_CRITICAL_UPS,
 				      ups_low_actions);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_ups_computer",
-				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
+				      GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT);
 	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
 	set_idle_hscale_stops (prefs, "hscale_ups_computer", delay);
 }
@@ -1035,10 +1094,12 @@ prefs_setup_general (GpmPrefs *prefs)
 				      GPM_CONF_BUTTON_SUSPEND,
 				      suspend_button_actions);
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_general_ambient",
-				  GPM_CONF_DISPLAY_STATE_CHANGE);
-	/* for now, hide */
-	widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_general_ambient");
-	gtk_widget_hide_all (widget);
+				  GPM_CONF_AMBIENT_ENABLE);
+
+	if (prefs->priv->has_ambient == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_general_ambient");
+		gtk_widget_hide_all (widget);
+	}
 
 	if (prefs->priv->has_button_suspend == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_general_suspend");
@@ -1053,13 +1114,13 @@ prefs_setup_general (GpmPrefs *prefs)
 static void
 gpm_prefs_init (GpmPrefs *prefs)
 {
-	GtkWidget    *main_window;
-	GtkWidget    *widget;
+	GtkWidget *main_window;
+	GtkWidget *widget;
+	gint caps;
 
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
-	prefs->priv->hal = gpm_hal_new ();
-	prefs->priv->cpufreq = gpm_cpufreq_new ();
+	prefs->priv->hal_cpufreq = hal_gcpufreq_new ();
 
 	prefs->priv->screensaver = gpm_screensaver_new ();
 	g_signal_connect (prefs->priv->screensaver, "gs-delay-changed",
@@ -1069,28 +1130,23 @@ gpm_prefs_init (GpmPrefs *prefs)
 	g_signal_connect (prefs->priv->conf, "value-changed",
 			  G_CALLBACK (conf_key_changed_cb), prefs);
 
-	prefs->priv->has_lcd = gpm_hal_num_devices_of_capability (prefs->priv->hal, "laptop_panel") > 0;
-	prefs->priv->has_batteries = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "battery",
-							"battery.type",
-							"primary") > 0;
-	prefs->priv->has_ups = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "battery",
-							"battery.type",
-							"ups") > 0;
-	prefs->priv->has_button_lid = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
-							"button.type",
-							"lid") > 0;
-	prefs->priv->has_button_suspend = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
-							"button.type",
-							"suspend") > 0;
-	prefs->priv->can_suspend = gpm_dbus_method_bool ("AllowedSuspend");
-	prefs->priv->can_hibernate = gpm_dbus_method_bool ("AllowedHibernate");
+	caps = gpm_dbus_method_int ("GetPreferencesOptions");
+	prefs->priv->has_batteries = ((caps & GPM_PREFS_SERVER_BATTERY) > 0);
+	prefs->priv->has_ups = ((caps & GPM_PREFS_SERVER_UPS) > 0);
+	prefs->priv->has_lcd = ((caps & GPM_PREFS_SERVER_BACKLIGHT) > 0);
+	prefs->priv->has_ambient = ((caps & GPM_PREFS_SERVER_AMBIENT) > 0);
+	prefs->priv->has_button_lid = ((caps & GPM_PREFS_SERVER_LID) > 0);
+	prefs->priv->has_button_suspend = TRUE;
+	prefs->priv->can_suspend = gpm_dbus_method_bool ("CanSuspend");
+	prefs->priv->can_hibernate = gpm_dbus_method_bool ("CanHibernate");
+	gpm_debug ("caps=%i", caps);
 
 	/* only enable cpufreq stuff if we have the hardware */
-	if (prefs->priv->cpufreq) {
-		gpm_cpufreq_get_governors (prefs->priv->cpufreq,
-					       &prefs->priv->cpufreq_types);
+	if (prefs->priv->hal_cpufreq) {
+		hal_gcpufreq_get_governors (prefs->priv->hal_cpufreq,
+					    &prefs->priv->cpufreq_types);
 	} else {
-		prefs->priv->cpufreq_types = GPM_CPUFREQ_NOTHING;
+		prefs->priv->cpufreq_types = LIBHAL_CPUFREQ_NOTHING;
 	}
 
 	prefs->priv->glade_xml = glade_xml_new (GPM_DATA "/gpm-prefs.glade", NULL, NULL);
@@ -1144,11 +1200,8 @@ gpm_prefs_finalize (GObject *object)
 	if (prefs->priv->screensaver) {
 		g_object_unref (prefs->priv->screensaver);
 	}
-	if (prefs->priv->cpufreq) {
-		g_object_unref (prefs->priv->cpufreq);
-	}
-	if (prefs->priv->hal) {
-		g_object_unref (prefs->priv->hal);
+	if (prefs->priv->hal_cpufreq) {
+		g_object_unref (prefs->priv->hal_cpufreq);
 	}
 
 	G_OBJECT_CLASS (gpm_prefs_parent_class)->finalize (object);
