@@ -28,13 +28,13 @@
 #include <stdlib.h>
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-help.h>
+#include <gtk/gtk.h>
 
 #if HAVE_GTKUNIQUE
 #include <gtkunique/gtkunique.h>
 #endif
 
+#include "gpm-common.h"
 #include "gpm-prefs.h"
 #include "gpm-debug.h"
 #include "gpm-prefs-core.h"
@@ -48,14 +48,7 @@
 static void
 gpm_prefs_help_cb (GpmPrefs *prefs)
 {
-	GError *error = NULL;
-
-	gnome_help_display_with_doc_id (NULL, "gnome-power-manager",
-					"gnome-power-manager.xml", NULL, &error);
-	if (error != NULL) {
-		gpm_warning (error->message);
-		g_error_free (error);
-	}
+	gpm_help_display ("preferences");
 }
 
 /**
@@ -102,7 +95,6 @@ main (int argc, char **argv)
 {
 	gboolean verbose = FALSE;
 	GOptionContext *context;
- 	GnomeProgram *program;
 	GpmPrefs *prefs = NULL;
 	GMainLoop *loop;
 #if HAVE_GTKUNIQUE
@@ -124,8 +116,6 @@ main (int argc, char **argv)
 
 	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
 
-	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
-
 #if HAVE_GTKUNIQUE
 	/* FIXME: We don't need to get the startup id once we can
 	 * depend on gtk+-2.12.  Until then we must get it BEFORE
@@ -134,15 +124,11 @@ main (int argc, char **argv)
 	 */
 	startup_id = g_getenv ("DESKTOP_STARTUP_ID");
 #endif
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (FALSE));
+	g_option_context_parse (context, &argc, &argv, NULL);
 
-	program = gnome_program_init (argv[0], VERSION, LIBGNOMEUI_MODULE,
-			    argc, argv,
-			    GNOME_PROGRAM_STANDARD_PROPERTIES,
-			    GNOME_PARAM_GOPTION_CONTEXT, context,
-			    GNOME_PARAM_HUMAN_READABLE_NAME,
-			    _("Power Preferences"),
-			    NULL);
-
+	gtk_init (&argc, &argv);
 	gpm_debug_init (verbose);
 
 #if HAVE_GTKUNIQUE
@@ -192,7 +178,6 @@ main (int argc, char **argv)
 #if HAVE_GTKUNIQUE
 	g_object_unref (uniqueapp);
 #endif
-	g_object_unref (program);
 /* seems to not work...
 	g_option_context_free (context); */
 
